@@ -30,6 +30,7 @@
 
 #include <boost/algorithm/string/predicate.hpp>
 #include <ignition/math/Rand.hh>
+#include <ignition/math/SemanticVersion.hh>
 
 #include <gazebo/gazebo_config.h>
 
@@ -37,7 +38,7 @@
 #include <ignition/msgs/stringmsg.pb.h>
 
 #include "ignition/common/Profiler.hh"
-#include <ignition/common/URI.hh>
+#include "ignition/common/URI.hh"
 #include "gazebo/common/FuelModelDatabase.hh"
 
 #include "gazebo/transport/Node.hh"
@@ -179,6 +180,7 @@ void World::Load(sdf::ElementPtr _sdf)
 {
   this->dataPtr->loaded = false;
   this->dataPtr->sdf = _sdf;
+  common::convertToFullPaths(this->dataPtr->sdf);
 
   // Create a DOM object to compute the resolved initial pose (with frame
   // semantics)
@@ -186,11 +188,18 @@ void World::Load(sdf::ElementPtr _sdf)
   sdf::Errors errors = this->dataPtr->worldSDFDom->Load(_sdf);
 
   // Print errors and load the parts that worked.
-  for (const auto &error : errors)
+  auto sdfVersion =
+    ignition::math::SemanticVersion(_sdf->OriginalVersion());
+  // Only print out errors if the original SDFormat version does not support
+  // frame semantics
+  if (sdfVersion >= ignition::math::SemanticVersion(1, 7))
   {
-    if (common::isSdfFrameSemanticsError(error))
+    for (const auto &error : errors)
     {
-      gzerr << error << "\n";
+      if (common::isSdfFrameSemanticsError(error))
+      {
+        gzerr << error << "\n";
+      }
     }
   }
 
@@ -208,6 +217,18 @@ void World::Load(sdf::ElementPtr _sdf)
       msgs::SceneFromSDF(this->dataPtr->sdf->GetElement("scene")));
   this->dataPtr->sceneMsg.set_name(this->Name());
 
+  if (this->dataPtr->sdf->GetElement("scene")->
+      HasElement("ignition:shadow_caster_material_name"))
+  {
+    this->dataPtr->shadowCasterMaterialName =
+      this->dataPtr->sdf->GetElement("scene")->
+        Get<std::string>("ignition:shadow_caster_material_name");
+  }
+  else
+  {
+    this->dataPtr->shadowCasterMaterialName = "Gazebo/shadow_caster";
+  }
+
   // The period at which messages are processed
   this->dataPtr->processMsgsPeriod = common::Time(0, 200000000);
 
@@ -275,6 +296,14 @@ void World::Load(sdf::ElementPtr _sdf)
         << std::endl;
   }
 
+  std::string shadowCasterService("/shadow_caster_material_name");
+  if (!this->dataPtr->ignNode.Advertise(shadowCasterService,
+      &World::ShadowCasterService, this))
+  {
+    gzerr << "Error advertising service [" << shadowCasterService << "]"
+        << std::endl;
+  }
+
   // This should come before loading of entities
   sdf::ElementPtr physicsElem = this->dataPtr->sdf->GetElement("physics");
 
@@ -1143,7 +1172,7 @@ BasePtr World::BaseByName(const std::string &_name) const
 ModelPtr World::ModelById(unsigned int _id) const
 {
   return boost::dynamic_pointer_cast<Model>(
-      this->dataPtr->rootElement->GetById(_id));
+      this->dataPtr->rootElement->GetByIdRecursive(_id));
 }
 
 //////////////////////////////////////////////////
@@ -2091,6 +2120,7 @@ void World::ProcessFactoryMsgs()
 
   for (auto const &factoryMsg : factoryMsgsCopy)
   {
+
     this->dataPtr->factorySDF->Clear();
 
     if (factoryMsg.has_sdf() && !factoryMsg.sdf().empty())
@@ -2122,9 +2152,11 @@ void World::ProcessFactoryMsgs()
 
       if (!sdf::readFile(filename, this->dataPtr->factorySDF))
       {
-        gzerr << "Unable to read sdf file.\n";
+        gzerr << "Unable to read sdf file [" << filename << "]\n";
         continue;
       }
+
+      common::convertToFullPaths(this->dataPtr->factorySDF->Root());
     }
     else if (factoryMsg.has_clone_model_name())
     {
@@ -3325,3 +3357,10 @@ bool World::PluginInfoService(const ignition::msgs::StringMsg &_req,
 
   return false;
 }
+
+//////////////////////////////////////////////////
+bool World::ShadowCasterService(ignition::msgs::StringMsg &_res)
+{
+  _res.set_data(this->dataPtr->shadowCasterMaterialName.c_str());
+  return true;
+}
